Program Unix;

uses
 Objects, Memory, tpString, Dos, RevPos, GetID;

type
 PIdxType = ^TIdxType;
 TIdxType = record
  PTopic, PFName: PString;
 end;

 PIdxColl = ^TIdxColl;     {Topic index collection}
 TIdxColl = Object(TSortedCollection)
  function Compare(Key1, Key2: Pointer): Integer; virtual;
  procedure FreeItem(Item: Pointer); virtual;
 end;

const
 TermChars = [' ', '.', ':', ',', '-', ''];

var
 PIdx: PIdxColl;

function TIdxColl.Compare(Key1, Key2: Pointer): Integer;
var
 C: CompareType;
begin
 C := CompUCString(PIdxType(Key1)^.PTopic^, PIdxType(Key2)^.PTopic^);
 if C = Greater then Compare := 1
 else if C = Less then Compare := -1
      else Compare := 0;
end;

procedure TIdxColl.FreeItem(Item: Pointer);
begin
 if Item <> Nil then
  begin
   DisposeStr(PIdxType(Item)^.PTopic);
   DisposeStr(PIdxType(Item)^.PFName);
   FreeMem(Item, SizeOf(TIdxType));
  end;
end;

procedure ReadIndex(const HlpPath: String);
var
 SR: SearchRec;
 T: Text;
 S: String;
 Ps: Byte;
 PItem: PIdxType;
begin
 FindFirst(HlpPath + '*.*', $2F, SR);
 while DosError = 0 do
 begin
   Assign(T, HlpPath + SR.Name);
   Reset(T);
   if IOResult = 0 then
    begin
     repeat
      ReadLn(T, S);
     until (S <> '') or eof(T);
     Ps := Pos('   ', S);
     S[0] := Char(Ps-1);
     PItem := MemAlloc(SizeOf(TIdxType));
     if PItem <> Nil then
      begin
       PItem^.PTopic := NewStr(S);
       PItem^.PFName := NewStr(SR.Name);
       PIdx^.Insert(PItem);
       Write(PIdx^.Count, ' topics founded   '#13);
      end;
    end;
   Close(T);
   InOutRes := 0;
   FindNext(SR);
 end;
end;

function Str2Topic(const S: String): String;
var
 i: Byte;
 SS: String;
begin
 SS := '';
 for i := 1 to Length(S) do
  if S[i] in ['(', ')'] then SS[i] := '_'
  else SS[i] := S[i];
 SS[0] := S[0];
 Str2Topic := SS;
end;

function RefString(S: String; Index: Boolean): String;
var
 pb, pe: Byte;
 i: Integer;
 PItem: PIdxType;
 PStr: PString;
 Find: Boolean;
 SS, SSS: String[128];
begin
 S := ReplaceChar(':', '', S);
 S := ReplaceChar('{', '', S);
 S := ReplaceChar('}', '', S);
 S := DelChar('', S);
 S := DelChar('', S);
 S := Detab(S,8);
 PItem := @PStr;
 pe := Length(S);
 repeat
  while (S[pe] in TermChars) and (pe > 0) do dec(pe);
  pb := pe;
  while not (S[pb] in TermChars) and (pb > 0) do dec(pb);
  SS := Copy(S, pb+1, pe-pb);
  PStr := @SS;
  Find := PIdx^.Search(PItem, i);
  if Find or (Index and (Length(SS) > 1) and
     (Pos(StUpCase(SS), StUpCase(PIdxType(PIdx^.At(i))^.PTopic^)) <> 0)) then
   begin
    SSS := Str2Topic(PIdxType(PIdx^.At(i))^.PTopic^);
    Delete(S, pb + 1, pe-pb);
    Insert('{'+ SS + ':' + SSS + '}', S, pb + 1);
   end;
  pe := pb;
 until SS = '';
 RefString := S;
end;

procedure CreateIndex(const IIName, IOName: String);
var
 T, O: Text;
 S: String;
 i: Integer;
 Lines: Word;
 Section: Byte;
 PItem: PIdxType;
begin
 Assign(O, IOName);
 Rewrite(O);
 Assign(T, IIName);
 Reset(T);
 while (IOResult = 0) and not eof(T) do
  begin
   ReadLn(T, S);
   WriteLn(O, RefString(S, True));
  end;
 Close(O);
 InOutRes := 0;
 Close(T);
 InOutRes := 0;
end;

procedure CreateHelp(const HlpPath, HName: String);
var
 T, O: Text;
 S: String;
 i: Integer;
 Lines: Word;
 Section: Byte;
 PItem: PIdxType;
begin
 Assign(O, HName);
 Rewrite(O);
 WriteLn(O, '.topic About');
 WriteLn(O, '.include index.hls');
 WriteLn(O, '.topic Index');
 for i := 0 to PIdx^.Count - 1 do
  begin
   PItem := PIdx^.At(i);
   WriteLn(O, ' {', PItem^.PTopic^, ':', Str2Topic(PItem^.PTopic^), '}');
  end;

 Close(O);
 Append(O);
 for i := 0 to PIdx^.Count - 1 do
  begin
   PItem := PIdx^.At(i);
   Assign(T, HlpPath + PItem^.PFName^);
   Reset(T);
   if IOResult = 0 then
    begin
     Write(i+1, ' topics created   '#13);
     WriteLn(O, '.topic ', Str2Topic(PItem^.PTopic^));
     WriteLn(O, '{', PItem^.PTopic^, ':Color_2}');
     WriteLn(O,'');
     repeat
      ReadLn(T, S);
     until (S <> '') or eof(T);
     Section := 1;
     Lines := 1;
     ReadLn(T, S);
     while (IOResult = 0) and not eof(T) do
      begin
       ReadLn(T, S);
       WriteLn(O, RefString(S, False));
       Inc(Lines);
       if Lines = 400 then
        begin
         WriteLn(O, '');
         WriteLn(O, '{See next section:S', Section,
                    '_', Str2Topic(PItem^.PTopic^), '}');
         WriteLn(O, '.topic S', Section,
                    '_', Str2Topic(PItem^.PTopic^));
         Inc(Section);
         Lines := 0;
        end;
      end;
    end;
   Close(T);
   InOutRes := 0;
  end;
 Close(O);
 InOutRes := 0;
end;

begin
 PIdx := New(PIdxColl, Init(10, 10));
 ReadIndex('');
 WriteLn;
 CreateIndex('..\index', '..\INDEX.HLS');
 CreateHelp('', '..\UNIX.HLS');
 WriteLn;
 Dispose(PIdx, Done);
end.
